import pytest
import datetime

@pytest.mark.parametrize('field, expected_value', [
    ('PRODID', '-//Plönë.org//NONSGML plone.app.event//EN'),
    ('X-WR-CALDESC', 'test non ascii: äöü ÄÖÜ €'),
])
def test_calendar_from_ical_respects_unicode(field, expected_value, calendars):
    cal = calendars.calendar_with_unicode
    assert cal[field].to_ical().decode('utf-8') == expected_value

@pytest.mark.parametrize('test_input, field, expected_value', [
    ('event_with_unicode_fields', 'SUMMARY', 'Non-ASCII Test: ÄÖÜ äöü €'),
    ('event_with_unicode_fields', 'DESCRIPTION', 'icalendar should be able to handle non-ascii: €äüöÄÜÖ.'),
    ('event_with_unicode_fields', 'LOCATION', 'Tribstrül'),
    # Non-unicode characters in summary
    # https://github.com/collective/icalendar/issues/64
    ('issue_64_event_with_non_ascii_summary', 'SUMMARY', 'åäö'),
    # Unicode characters in summary
    ('issue_64_event_with_ascii_summary', 'SUMMARY', 'abcdef'),
])
def test_event_from_ical_respects_unicode(test_input, field, expected_value, events):
    event = events[test_input]
    assert event[field].to_ical().decode('utf-8') == expected_value

@pytest.mark.parametrize('test_input, expected_output', [
    # chokes on umlauts in ORGANIZER
    # https://github.com/collective/icalendar/issues/101
    ('issue_101_icalendar_chokes_on_umlauts_in_organizer', 'acme, ädmin'),
    ('event_with_unicode_organizer', 'Джон Доу'),
])
def test_events_parameter_unicoded(events, test_input, expected_output):
    assert events[test_input]['ORGANIZER'].params['CN'] == expected_output

def test_parses_event_with_non_ascii_tzid_issue_237(calendars, in_timezone):
    """Issue #237 - Fail to parse timezone with non-ascii TZID
    see https://github.com/collective/icalendar/issues/237
    """
    start = calendars.issue_237_fail_to_parse_timezone_with_non_ascii_tzid.walk('VEVENT')[0].decoded('DTSTART')
    expected = in_timezone(datetime.datetime(2017, 5, 11, 13, 30), 'America/Sao_Paulo')
    assert not calendars.issue_237_fail_to_parse_timezone_with_non_ascii_tzid.errors
    assert start == expected

def test_parses_timezone_with_non_ascii_tzid_issue_237(timezones):
    """Issue #237 - Fail to parse timezone with non-ascii TZID
    see https://github.com/collective/icalendar/issues/237
    """
    assert timezones.issue_237_brazilia_standard['tzid'] == '(UTC-03:00) Brasília'

@pytest.mark.parametrize('timezone_name', ['standard', 'daylight'])
def test_parses_timezone_with_non_ascii_tzname_issue_273(timezones, timezone_name):
    """Issue #237 - Fail to parse timezone with non-ascii TZID
    see https://github.com/collective/icalendar/issues/237
    """
    assert timezones.issue_237_brazilia_standard.walk(timezone_name)[0]['TZNAME'] == f'Brasília {timezone_name}'

def test_broken_property(calendars):
    """
    Test if error messages are encoded properly.
    """
    for event in calendars.broken_ical.walk('vevent'):
        assert len(event.errors) == 1, 'Not the right amount of errors.'
        error = event.errors[0][1]
        assert error.startswith('Content line could not be parsed into parts')

def test_apple_xlocation(calendars):
    """
    Test if we support base64 encoded binary data in parameter values.
    """
    for event in calendars.x_location.walk('vevent'):
        assert len(event.errors) == 0, 'Got too many errors'



"""Some parameters are specified as default by the RFC 5545.

These tests make sure that these parameter values are not added to the
properties.

Example:

       DTSTART;VALUE=DATE-TIME:20190616T050000Z
equals DTSTART:20190616T050000Z
"""
import pytest
from icalendar import Event
from datetime import datetime


@pytest.mark.parametrize("attr", [
    "DTSTART",
    "DTEND",
    "DTSTAMP",
])
def test_datetime_in_event(attr):
    """Check that the "VALUE=DATE-TIME" is absent because not needed."""
    event = Event()
    event.add(attr, datetime(2022, 10, 13, 9, 16, 42))
    ics = event.to_ical()
    assert b"VALUE=DATE-TIME" not in ics




from datetime import datetime
from icalendar.parser_tools import to_unicode
from icalendar.prop import vDatetime
from icalendar.prop import vText
from string import ascii_letters
from string import digits

import random


class UIDGenerator:
    """If you are too lazy to create real uid's.

    """
    chars = list(ascii_letters + digits)

    @staticmethod
    def rnd_string(length=16):
        """Generates a string with random characters of length.
        """
        return ''.join([random.choice(UIDGenerator.chars) for _ in range(length)])

    @staticmethod
    def uid(host_name='example.com', unique=''):
        """Generates a unique id consisting of:
            datetime-uniquevalue@host.
        Like:
            20050105T225746Z-HKtJMqUgdO0jDUwm@example.com
        """
        host_name = to_unicode(host_name)
        unique = unique or UIDGenerator.rnd_string()
        today = to_unicode(vDatetime(datetime.today()).to_ical())
        return vText(f'{today}-{unique}@{host_name}')



import pytest
from icalendar import Calendar, Event, Parameters, vCalAddress

import unittest
import icalendar
import re

@pytest.mark.parametrize('parameter, expected', [
    # Simple parameter:value pair
    (Parameters(parameter1='Value1'), b'PARAMETER1=Value1'),
    # Parameter with list of values must be separated by comma
    (Parameters({'parameter1': ['Value1', 'Value2']}), b'PARAMETER1=Value1,Value2'),
    # Multiple parameters must be separated by a semicolon
    (Parameters({'RSVP': 'TRUE', 'ROLE': 'REQ-PARTICIPANT'}), b'ROLE=REQ-PARTICIPANT;RSVP=TRUE'),
    # Parameter values containing ',;:' must be double quoted
    (Parameters({'ALTREP': 'http://www.wiz.org'}), b'ALTREP="http://www.wiz.org"'),
    # list items must be quoted separately
    (Parameters({'MEMBER': ['MAILTO:projectA@host.com',
                                   'MAILTO:projectB@host.com']}),
     b'MEMBER="MAILTO:projectA@host.com","MAILTO:projectB@host.com"'),
    (Parameters({'parameter1': 'Value1',
                        'parameter2': ['Value2', 'Value3'],
                        'ALTREP': ['http://www.wiz.org', 'value4']}),
     b'ALTREP="http://www.wiz.org",value4;PARAMETER1=Value1;PARAMETER2=Value2,Value3'),
    # Including empty strings
    (Parameters({'PARAM': ''}), b'PARAM='),
    # We can also parse parameter strings
    (Parameters({'MEMBER': ['MAILTO:projectA@host.com',
                                   'MAILTO:projectB@host.com']}),
     b'MEMBER="MAILTO:projectA@host.com","MAILTO:projectB@host.com"'),
    # We can also parse parameter strings
    (Parameters({'PARAMETER1': 'Value1',
                 'ALTREP': ['http://www.wiz.org', 'value4'],
                 'PARAMETER2': ['Value2', 'Value3']}),
     b'ALTREP="http://www.wiz.org",value4;PARAMETER1=Value1;PARAMETER2=Value2,Value3'),
])
def test_parameter_to_ical_is_inverse_of_from_ical(parameter, expected):
    assert parameter.to_ical() == expected
    assert Parameters.from_ical(expected.decode('utf-8')) == parameter

def test_parse_parameter_string_without_quotes():
    assert Parameters.from_ical('PARAM1=Value 1;PARA2=Value 2') == Parameters({'PARAM1': 'Value 1', 'PARA2': 'Value 2'})

def test_parametr_is_case_insensitive():
    parameter = Parameters(parameter1='Value1')
    assert parameter['parameter1'] == parameter['PARAMETER1'] == parameter['PaRaMeTer1']

def test_parameter_keys_are_uppercase():
    parameter = Parameters(parameter1='Value1')
    assert list(parameter.keys()) == ['PARAMETER1']

@pytest.mark.parametrize('cn_param, cn_quoted', [
    # not double-quoted
    ('Aramis', 'Aramis'),
    # if a space is present - enclose in double quotes
    ('Aramis Alameda', '"Aramis Alameda"'),
    # a single quote in parameter value - double quote the value
    ('Aramis d\'Alameda', '"Aramis d\'Alameda"'),
    ('Арамис д\'Аламеда', '"Арамис д\'Аламеда"'),
    # double quote is replaced with single quote
    ('Aramis d\"Alameda', '"Aramis d\'Alameda"'),
])
def test_quoting(cn_param, cn_quoted):
    event = Event()
    attendee = vCalAddress('test@example.com')
    attendee.params['CN'] = cn_param
    event.add('ATTENDEE', attendee)
    assert f'ATTENDEE;CN={cn_quoted}:test@example.com' in event.to_ical().decode('utf-8')

class TestPropertyParams(unittest.TestCase):

    def test_property_params(self):
        # Property parameters with values containing a COLON character, a
        # SEMICOLON character or a COMMA character MUST be placed in quoted
        # text.
        cal_address = vCalAddress('mailto:john.doe@example.org')
        cal_address.params["CN"] = "Doe, John"
        ical = Calendar()
        ical.add('organizer', cal_address)

        ical_str = Calendar.to_ical(ical)
        exp_str = b"""BEGIN:VCALENDAR\r\nORGANIZER;CN="Doe, John":"""\
                  b"""mailto:john.doe@example.org\r\nEND:VCALENDAR\r\n"""

        self.assertEqual(ical_str, exp_str)

        # other way around: ensure the property parameters can be restored from
        # an icalendar string.
        ical2 = Calendar.from_ical(ical_str)
        self.assertEqual(ical2.get('ORGANIZER').params.get('CN'), 'Doe, John')

    def test_parse_and_access_property_params(self):
        """Parse an ics string and access some property parameters then.
        This is a follow-up of a question received per email.

        """
        ics = """BEGIN:VCALENDAR
VERSION:2.0
PRODID://RESEARCH IN MOTION//BIS 3.0
METHOD:REQUEST
BEGIN:VEVENT
SEQUENCE:2
X-RIM-REVISION:0
SUMMARY:Test meeting from BB
X-MICROSOFT-CDO-ALLDAYEVENT:TRUE
CLASS:PUBLIC
ATTENDEE;PARTSTAT=NEEDS-ACTION;RSVP=TRUE;CN="RembrandXS":MAILTO:rembrand@xs4all.nl
ATTENDEE;PARTSTAT=NEEDS-ACTION;RSVP=TRUE;CN="RembrandDX":MAILTO:rembrand@daxlab.com
ATTENDEE;PARTSTAT=NEEDS-ACTION;RSVP=TRUE;CN="RembrandSB":MAILTO:rembspam@xs4all.nl
UID:XRIMCAL-628059586-522954492-9750559
DTSTART;VALUE=DATE:20120814
DTEND;VALUE=DATE:20120815
DESCRIPTION:Test meeting from BB
DTSTAMP:20120813T151458Z
ORGANIZER:mailto:rembrand@daxlab.com
END:VEVENT
END:VCALENDAR"""

        cal = icalendar.Calendar.from_ical(ics)
        event = cal.walk("VEVENT")[0]
        event['attendee'][0]
        self.assertEqual(event['attendee'][0].to_ical(),
                         b'MAILTO:rembrand@xs4all.nl')
        self.assertEqual(event['attendee'][0].params.to_ical(),
                         b'CN=RembrandXS;PARTSTAT=NEEDS-ACTION;RSVP=TRUE')
        self.assertEqual(event['attendee'][0].params['cn'], 'RembrandXS')

    def test_repr(self):
        """Test correct class representation.
        """
        it = Parameters(parameter1='Value1')
        self.assertTrue(
            re.match(r"Parameters\({u?'PARAMETER1': u?'Value1'}\)", str(it))
        )



import pytest

from icalendar import Event, Calendar

def test_ignore_exceptions_on_broken_events_issue_104(events):
    ''' Issue #104 - line parsing error in a VEVENT
    (which has ignore_exceptions). Should mark the event broken
    but not raise an exception.

    https://github.com/collective/icalendar/issues/104
    '''
    assert events.issue_104_mark_events_broken.is_broken # TODO: REMOVE FOR NEXT MAJOR RELEASE
    assert events.issue_104_mark_events_broken.errors == [(None, "Content line could not be parsed into parts: 'X': Invalid content line")]

def test_dont_ignore_exceptions_on_broken_calendars_issue_104(calendars):
    '''Issue #104 - line parsing error in a VCALENDAR
    (which doesn't have ignore_exceptions). Should raise an exception.
    '''
    with pytest.raises(ValueError):
        calendars.issue_104_broken_calendar

def test_rdate_dosent_become_none_on_invalid_input_issue_464(events):
    '''Issue #464 - [BUG] RDATE can become None if value is invalid
    https://github.com/collective/icalendar/issues/464
    '''
    assert events.issue_464_invalid_rdate.is_broken
    assert ('RDATE', 'Expected period format, got: 199709T180000Z/PT5H30M') in events.issue_464_invalid_rdate.errors
    assert not b'RDATE:None' in events.issue_464_invalid_rdate.to_ical()

@pytest.mark.parametrize('calendar_name', [
    'big_bad_calendar',
    'small_bad_calendar',
    'multiple_calendar_components',
    'pr_480_summary_with_colon',
])
def test_error_message_doesnt_get_too_big(calendars, calendar_name):
    with pytest.raises(ValueError) as exception:
        calendars[calendar_name]
    # Ignore part before first : for the test.
    assert len(str(exception).split(': ', 1)[1]) <= 100




'''Tests checking that parsing works'''
import pytest
import base64
from icalendar import Calendar, vRecur, vBinary, Event
from datetime import datetime
from icalendar.parser import Contentline, Parameters

@pytest.mark.parametrize('calendar_name', [
    # Issue #178 - A component with an unknown/invalid name is represented
    # as one of the known components, the information about the original
    # component name is lost.
    # https://github.com/collective/icalendar/issues/178 https://github.com/collective/icalendar/pull/180
    # Parsing of a nonstandard component
    'issue_178_component_with_invalid_name_represented',
    # Nonstandard component inside other components, also has properties
    'issue_178_custom_component_inside_other',
    # Nonstandard component is able to contain other components
    'issue_178_custom_component_contains_other',
])
def test_calendar_to_ical_is_inverse_of_from_ical(calendars, calendar_name):
    calendar = getattr(calendars, calendar_name)
    assert calendar.to_ical().splitlines() == calendar.raw_ics.splitlines()
    assert calendar.to_ical() == calendar.raw_ics

@pytest.mark.parametrize('raw_content_line, expected_output', [
    # Issue #142 - Multivalued parameters. This is needed for VCard 3.0.
    # see https://github.com/collective/icalendar/pull/142
    ('TEL;TYPE=HOME,VOICE:000000000', ('TEL', Parameters({'TYPE': ['HOME', 'VOICE']}), '000000000')),
    # Issue #143 - Allow dots in property names. Another vCard related issue.
    # see https://github.com/collective/icalendar/pull/143
    ('ITEMADRNULLTHISISTHEADRESS08158SOMECITY12345.ADR:;;This is the Adress 08; Some City;;12345;Germany', \
        ('ITEMADRNULLTHISISTHEADRESS08158SOMECITY12345.ADR', \
        Parameters(),\
        ';;This is the Adress 08; Some City;;12345;Germany')),
    ('ITEMADRNULLTHISISTHEADRESS08158SOMECITY12345.X-ABLABEL:', \
        ('ITEMADRNULLTHISISTHEADRESS08158SOMECITY12345.X-ABLABEL', \
        Parameters(), \
        ''))
])
def test_content_lines_parsed_properly(raw_content_line, expected_output):
    assert Contentline.from_ical(raw_content_line).parts() == expected_output


@pytest.mark.parametrize('timezone_info', [
    # General timezone aware dates in ical string
    (b'DTSTART;TZID=America/New_York:20130907T120000'),
    (b'DTEND;TZID=America/New_York:20130907T170000'),
    # Specific timezone aware exdates in ical string
    (b'EXDATE;TZID=America/New_York:20131012T120000'),
    (b'EXDATE;TZID=America/New_York:20131011T120000')
])
def test_timezone_info_present_in_ical_issue_112(events, timezone_info):
    '''Issue #112 - No timezone info on EXDATE

    https://github.com/collective/icalendar/issues/112
    '''
    timezone_info in events.issue_112_missing_tzinfo_on_exdate.to_ical()

def test_timezone_name_parsed_issue_112(events):
    '''Issue #112 - No timezone info on EXDATE

    https://github.com/collective/icalendar/issues/112
    '''
    assert events.issue_112_missing_tzinfo_on_exdate['exdate'][0].dts[0].dt.tzname() == 'EDT'

def test_issue_157_removes_trailing_semicolon(events):
    '''Issue #157 - Recurring rules and trailing semicolons

    https://github.com/collective/icalendar/pull/157
    '''
    recur = events.issue_157_removes_trailing_semicolon.decoded("RRULE")
    assert isinstance(recur, vRecur)
    assert recur.to_ical() == b'FREQ=YEARLY;BYDAY=1SU;BYMONTH=11'

@pytest.mark.parametrize('event_name', [
    # https://github.com/collective/icalendar/pull/100
    ('issue_100_transformed_doctests_into_unittests'),
    ('issue_184_broken_representation_of_period'),
    # PERIOD should be put back into shape
    'issue_156_RDATE_with_PERIOD',
    'issue_156_RDATE_with_PERIOD_list',
    'event_with_unicode_organizer',
])
def test_event_to_ical_is_inverse_of_from_ical(events, event_name):
    """Make sure that an event's ICS is equal to the ICS it was made from."""
    event = events[event_name]
    assert event.to_ical().splitlines() == event.raw_ics.splitlines()
    assert event.to_ical() == event.raw_ics

def test_decode_rrule_attribute_error_issue_70(events):
    # Issue #70 - e.decode("RRULE") causes Attribute Error
    # see https://github.com/collective/icalendar/issues/70
    recur = events.issue_70_rrule_causes_attribute_error.decoded('RRULE')
    assert isinstance(recur, vRecur)
    assert recur.to_ical() == b'FREQ=WEEKLY;UNTIL=20070619T225959;INTERVAL=1'

def test_description_parsed_properly_issue_53(events):
    '''Issue #53 - Parsing failure on some descriptions?

    https://github.com/collective/icalendar/issues/53
    '''
    assert b'July 12 at 6:30 PM' in events.issue_53_description_parsed_properly['DESCRIPTION'].to_ical()

def test_raises_value_error_for_properties_without_parent_pull_179():
        '''Found an issue where from_ical() would raise IndexError for
        properties without parent components.

        https://github.com/collective/icalendar/pull/179
        '''
        with pytest.raises(ValueError):
            Calendar.from_ical('VERSION:2.0')

def test_tzid_parsed_properly_issue_53(timezones):
    '''Issue #53 - Parsing failure on some descriptions?

    https://github.com/collective/icalendar/issues/53
    '''
    assert timezones.issue_53_tzid_parsed_properly['tzid'].to_ical() == b'America/New_York'
    
def test_timezones_to_ical_is_inverse_of_from_ical(timezones):
    '''Issue #55 - Parse error on utc-offset with seconds value
     see https://github.com/collective/icalendar/issues/55'''
    timezone = timezones['issue_55_parse_error_on_utc_offset_with_seconds']
    assert timezone.to_ical() == timezone.raw_ics

@pytest.mark.parametrize('date, expected_output', [
    (datetime(2012, 7, 16, 0, 0, 0), b'DTSTART:20120716T000000Z'),
    (datetime(2021, 11, 17, 15, 9, 15), b'DTSTART:20211117T150915Z')
])
def test_no_tzid_when_utc(utc, date, expected_output):
    '''Issue #58  - TZID on UTC DATE-TIMEs
       Issue #335 - UTC timezone identification is broken

    https://github.com/collective/icalendar/issues/58
    https://github.com/collective/icalendar/issues/335
    '''
    # According to RFC 2445: "The TZID property parameter MUST NOT be
    # applied to DATE-TIME or TIME properties whose time values are
    # specified in UTC.
    date = date.replace(tzinfo=utc)
    event = Event()
    event.add('dtstart', date)
    assert expected_output in event.to_ical()
    
def test_vBinary_base64_encoded_issue_82():
    '''Issue #82 - vBinary __repr__ called rather than to_ical from
                   container types
    https://github.com/collective/icalendar/issues/82
    '''
    b = vBinary('text')
    b.params['FMTTYPE'] = 'text/plain'
    assert b.to_ical() == base64.b64encode(b'text')

def test_creates_event_with_base64_encoded_attachment_issue_82(events):
    '''Issue #82 - vBinary __repr__ called rather than to_ical from
                   container types
    https://github.com/collective/icalendar/issues/82
    '''
    b = vBinary('text')
    b.params['FMTTYPE'] = 'text/plain'
    event = Event()
    event.add('ATTACH', b)
    assert event.to_ical() == events.issue_82_expected_output.raw_ics

@pytest.mark.parametrize('calendar_name', [
    # Issue #466 - [BUG] TZID timezone is ignored when forward-slash is used
    # https://github.com/collective/icalendar/issues/466
    'issue_466_respect_unique_timezone',
    'issue_466_convert_tzid_with_slash'
])
def test_handles_unique_tzid(calendars, in_timezone, calendar_name):
    calendar = calendars[calendar_name]
    start_dt = calendar.walk('VEVENT')[0]['dtstart'].dt
    end_dt = calendar.walk('VEVENT')[0]['dtend'].dt
    assert start_dt == in_timezone(datetime(2022, 10, 21, 20, 0, 0), 'Europe/Stockholm')
    assert end_dt == in_timezone(datetime(2022, 10, 21, 21, 0, 0), 'Europe/Stockholm')

@pytest.mark.parametrize('event_name, expected_cn, expected_ics', [
    ('event_with_escaped_characters', r'that, that; %th%%at%\ that:', 'это, то; that\\ %th%%at%:'),
    ('event_with_escaped_character1', r'Society, 2014', 'that'),
    ('event_with_escaped_character2', r'Society\ 2014', 'that'),
    ('event_with_escaped_character3', r'Society; 2014', 'that'),
    ('event_with_escaped_character4', r'Society: 2014', 'that'),
])
def test_escaped_characters_read(event_name, expected_cn, expected_ics, events):
    event = events[event_name]
    assert event['ORGANIZER'].params['CN'] == expected_cn
    assert event['ORGANIZER'].to_ical() == expected_ics.encode('utf-8')




from datetime import datetime

import pytest

def test_recurrence_properly_parsed(events):
    assert events.event_with_recurrence['rrule'] == {'COUNT': [100], 'FREQ': ['DAILY']}

@pytest.mark.parametrize('i, exception_date', [
    (0, datetime(1996, 4, 2, 1, 0)),
    (1, datetime(1996, 4, 3, 1, 0)),
    (2, datetime(1996, 4, 4, 1, 0))
])
def test_exdate_properly_parsed(events, i, exception_date, in_timezone):
    assert events.event_with_recurrence['exdate'].dts[i].dt == in_timezone(exception_date, 'UTC')

def test_exdate_properly_marshalled(events):
    actual = events.event_with_recurrence['exdate'].to_ical()
    assert actual == b'19960402T010000Z,19960403T010000Z,19960404T010000Z'

# TODO: DOCUMENT BETTER!
# In this case we have multiple EXDATE definitions, one per line.
# Icalendar makes a list out of this instead of zipping it into one
# vDDDLists object. Actually, this feels correct for me, as it also
# allows to define different timezones per exdate line - but client
# code has to handle this as list and not blindly expecting to be able
# to call event['EXDATE'].to_ical() on it:
def test_exdate_formed_from_exdates_on_multiple_lines_is_a_list(events):
    exdate = events.event_with_recurrence_exdates_on_different_lines['exdate']
    assert isinstance(exdate, list)

@pytest.mark.parametrize('i, exception_date, exception_date_ics', [
    (0, datetime(2012, 5, 29, 10, 0), b'20120529T100000'),
    (1, datetime(2012, 4, 3, 10, 0),  b'20120403T100000'),
    (2, datetime(2012, 4, 10, 10, 0), b'20120410T100000'),
    (3, datetime(2012, 5, 1, 10, 0),  b'20120501T100000'),
    (4, datetime(2012, 4, 17, 10, 0), b'20120417T100000')
])
def test_list_exdate_to_ical_is_inverse_of_from_ical(events, i, exception_date, exception_date_ics, in_timezone):
    exdate = events.event_with_recurrence_exdates_on_different_lines['exdate']
    assert exdate[i].dts[0].dt == in_timezone(exception_date, 'Europe/Vienna')
    assert exdate[i].to_ical() == exception_date_ics

